@@ -14,8 +14,8 @@ modules = [
     "2chen",
     "35photo",
     "3dbooru",
-    "420chan",
     "4chan",
+    "4chanarchives",
     "500px",
     "8chan",
     "8muses",
@@ -74,6 +74,7 @@ modules = [
     "instagram",
     "issuu",
     "itaku",
+    "itchio",
     "kabeuchi",
     "keenspot",
     "kemonoparty",
@@ -93,11 +94,9 @@ modules = [
     "mangapark",
     "mangasee",
     "mangoxo",
-    "mememuseum",
     "misskey",
     "myhentaigallery",
     "myportfolio",
-    "nana",
     "naver",
     "naverwebtoon",
     "newgrounds",
@@ -133,6 +132,7 @@ modules = [
     "seiga",
     "senmanga",
     "sexcom",
+    "shimmie2",
     "simplyhentai",
     "skeb",
     "slickpic",
@@ -156,6 +156,7 @@ modules = [
     "urlshortener",
     "vanillarock",
     "vichan",
+    "vipergirls",
     "vk",
     "vsco",
     "wallhaven",
