@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2018-2022 Mike Fährmann
+# Copyright 2018-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -11,7 +11,6 @@
 from .common import Extractor, Message
 from .. import text, util, exception
 from ..cache import cache, memcache
-from ..version import __version__
 from collections import defaultdict
 
 BASE_PATTERN = r"(?:https?://)?(?:www\.)?mangadex\.(?:org|cc)"
@@ -28,10 +27,10 @@ class MangadexExtractor(Extractor):
     archive_fmt = "{chapter_id}_{page}"
     root = "https://mangadex.org"
     _cache = {}
-    _headers = {"User-Agent": "gallery-dl/" + __version__}
 
     def __init__(self, match):
         Extractor.__init__(self, match)
+        self.session.headers["User-Agent"] = util.USERAGENT
         self.api = MangadexAPI(self)
         self.uuid = match.group(1)
 
@@ -86,6 +85,10 @@ class MangadexExtractor(Extractor):
         data["group"] = [group["attributes"]["name"]
                          for group in relationships["scanlation_group"]]
 
+        data["status"] = mattributes["status"]
+        data["tags"] = [tag["attributes"]["name"]["en"]
+                        for tag in mattributes["tags"]]
+
         return data
 
 
@@ -95,13 +98,13 @@ class MangadexChapterExtractor(MangadexExtractor):
     pattern = BASE_PATTERN + r"/chapter/([0-9a-f-]+)"
     test = (
         ("https://mangadex.org/chapter/f946ac53-0b71-4b5d-aeb2-7931b13c4aaa", {
-            "keyword": "86fb262cf767dac6d965cd904ad499adba466404",
+            "keyword": "e86128a79ebe7201b648f1caa828496a2878dc8f",
             #  "content": "50383a4c15124682057b197d40261641a98db514",
         }),
         # oneshot
         ("https://mangadex.org/chapter/61a88817-9c29-4281-bdf1-77b3c1be9831", {
             "count": 64,
-            "keyword": "6abcbe1e24eeb1049dc931958853cd767ee483fb",
+            "keyword": "d11ed057a919854696853362be35fc0ba7dded4c",
         }),
         # MANGA Plus (#1154)
         ("https://mangadex.org/chapter/74149a55-e7c4-44ea-8a37-98e879c1096f", {
@@ -109,7 +112,7 @@ class MangadexChapterExtractor(MangadexExtractor):
         }),
         # 'externalUrl', but still downloadable (#2503)
         ("https://mangadex.org/chapter/364728a4-6909-4164-9eea-6b56354f7c78", {
-            "count": 39,
+            "count": 0,  # 404
         }),
     )
 
@@ -127,7 +130,6 @@ class MangadexChapterExtractor(MangadexExtractor):
                 data["chapter"], data["chapter_minor"], data["_external_url"])
 
         yield Message.Directory, data
-        data["_http_headers"] = self._headers
 
         server = self.api.athome_server(self.uuid)
         chapter = server["chapter"]
@@ -159,6 +161,9 @@ class MangadexMangaExtractor(MangadexExtractor):
                 "language": str,
                 "artist"  : ["Arakawa Hiromu"],
                 "author"  : ["Arakawa Hiromu"],
+                "status"  : "completed",
+                "tags"    : ["Oneshot", "Historical", "Action",
+                             "Martial Arts", "Drama", "Tragedy"],
             },
         }),
         ("https://mangadex.cc/manga/d0c88e3b-ea64-4e07-9841-c1d2ac982f4a/", {
@@ -192,7 +197,7 @@ class MangadexAPI():
 
     def __init__(self, extr):
         self.extractor = extr
-        self.headers = extr._headers.copy()
+        self.headers = {}
 
         self.username, self.password = self.extractor._get_auth_info()
         if not self.username:
