@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2016-2022 Mike Fährmann, Leonardo Taccari
+# Copyright 2016-2023 Mike Fährmann, Leonardo Taccari
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,8 +9,7 @@
 """Extractors for https://www.slideshare.net/"""
 
 from .common import GalleryExtractor
-from .. import text
-import json
+from .. import text, util
 
 
 class SlidesharePresentationExtractor(GalleryExtractor):
@@ -59,7 +58,7 @@ class SlidesharePresentationExtractor(GalleryExtractor):
         # mobile URL
         (("https://www.slideshare.net"
           "/mobile/uqudent/introduction-to-fixed-prosthodontics"), {
-            "url": "59993ad7b0cb93c73011547eedcd02c622649e9d",
+            "url": "43eda2adf4dd221a251c8df794dfb82649e94647",
         }),
     )
 
@@ -72,14 +71,14 @@ class SlidesharePresentationExtractor(GalleryExtractor):
     def metadata(self, page):
         extr = text.extract_from(page)
         descr = extr('<meta name="description" content="', '"')
-        title = extr('<span class="j-title-breadcrumb">', '</span>')
-        published = extr('<div class="metadata-item">', '</div>')
         comments = extr('content="UserComments:', '"')
         likes = extr('content="UserLikes:', '"')
         views = extr('content="UserPageVisits:', '"')
+        title = extr('<span class="j-title-breadcrumb">', '</span>')
+        published = extr('<div class="metadata-item">', '</div>')
 
         if descr.endswith("…"):
-            alt_descr = extr('id="slideshow-description-text"', '</p>')
+            alt_descr = extr('slideshow-description-text"', '</p>')
             if alt_descr:
                 descr = text.remove_html(alt_descr.partition(">")[2]).strip()
 
@@ -97,7 +96,7 @@ class SlidesharePresentationExtractor(GalleryExtractor):
 
     @staticmethod
     def images(page):
-        data = json.loads(text.extract(
+        data = util.json_loads(text.extract(
             page, "xtend(true, slideshare_object.slideshow_config, ", ");")[0])
 
         # useing 'stripped_title' here is technically wrong, but it works all
